# Ruteo Dinmico por Vector Distancia
#
#                     n2  
#                  .       . 
#               .             .
#            .                   .
# n0--------n1                    n4-------n5
#            .                   .
#               .             .
#                  .       . 
#                     n3  


set ns [new Simulator]
set fin_sim 1000

# Traza general
set f [open salida.tr w]
$ns trace-all $f

# Traza para el nam
# set nf [open out.nam w]
# $ns namtrace-all $nf

set lamda 50.0
set mu 80.0


# Indica que se usar ruteo dinmico por vector distancia
$ns rtproto DV 

# crea un archivo para registrar tamaos de cola
set tq [open cola_2_4 w]
set tq1 [open cola_3_4 w]
set tq2 [open cola_0_1 w]
set tq3 [open cola_4_5 w]
#
# Se genera la topologa de la red
#

# crea los nodos y especifica el tipo de ruteo 

set n0 [$ns node]
set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]
set n4 [$ns node]
set n5 [$ns node]

# se habilita al nodo 2 a utilizar multipath

$n1 set multiPath_ 1

# crea los enlaces
$ns duplex-link $n0 $n1 100kb 0ms DropTail
$ns duplex-link $n1 $n2 400kb 0ms DropTail
$ns duplex-link $n1 $n3 400kb 0ms DropTail      
$ns duplex-link $n3 $n4 100kb 0ms DropTail
$ns duplex-link $n2 $n4 100kb 0ms DropTail
$ns duplex-link $n4 $n5 100kb 0ms DropTail

# asigna costos a los enlaces
$ns cost $n0 $n1 1
$ns cost $n1 $n2 1
$ns cost $n1 $n3 1
$ns cost $n3 $n4 1
$ns cost $n2 $n4 1
$ns cost $n4 $n5 1


# crea un agente UDP y lo asigna al nodo n0
set fuente [new Agent/UDP]
$ns attach-agent $n0 $fuente


# crea un receptor y lo asigna a n5.
set sink [new Agent/Null] 
$ns attach-agent $n5 $sink


# conecta la fuente y el receptor

$ns connect $fuente $sink

# Generamos aleatoriamente tiempos de arribo de paquetes y tamaos de paquetes de acuerdo
# a una distribucin exponencial.

set InterArrivalTime [new RandomVariable/Exponential]
$InterArrivalTime set avg_ [expr 1/$lamda]
set pktSize [new RandomVariable/Exponential]
$pktSize set avg_ [expr 100000.0/(8*$mu)]


# crea monitores de cola
$ns trace-queue $n2 $n4
$ns trace-queue $n3 $n4
$ns trace-queue $n0 $n1
$ns trace-queue $n4 $n5
set qmon [$ns monitor-queue $n2 $n4 ""]
set qmon1 [$ns monitor-queue $n3 $n4 ""]
set qmon2 [$ns monitor-queue $n0 $n1 ""]
set qmon3 [$ns monitor-queue $n4 $n5 ""]

# Procedimiento que se llama periodicamente (cada time seg) para registrar el tamao de la cola.


       
proc record {} {
	global ns qmon qmon1 qmon2 qmon3 tq tq1 tq2 tq3      
	set now [$ns now]
	set time 0.01
	set tamcol [eval $qmon set size_]
	set tamcol1 [eval $qmon1 set size_]
	set tamcol2 [eval $qmon2 set size_]
	set tamcol3 [eval $qmon3 set size_]
	puts $tq "$now $tamcol"
	puts $tq1 "$now $tamcol1"
	puts $tq2 "$now $tamcol2"
	puts $tq3 "$now $tamcol3"
	
	$ns at [expr $now+$time] "record"
}


# Procedimiento de finalizacin

proc finish {} {
      global ns tq tq1 tq2 tq3 qmon qmon1 qmon2 qmon3 fin_sim f nf lamda mu
      
      $ns flush-trace
      
      close $f     
#     close $nf 
      close $tq
      close $tq1
      close $tq2
      close $tq3
          
           
      exec clear 
          
      set pktsInt [$qmon get-pkts-integrator]
      set suma [expr [$pktsInt set sum_]]
      set avg_queue [expr $suma/$fin_sim]
      puts [format "N promedio de paquetes en la cola 2----4 = %9.5f paquetes" $avg_queue]
      puts ""
      
      set pktsInt1 [$qmon1 get-pkts-integrator]
      set suma1 [expr [$pktsInt1 set sum_]]
      set avg_queue1 [expr $suma1/$fin_sim]
      puts [format "N promedio de paquetes en la cola 3----4 = %9.5f paquetes" $avg_queue1]
      puts ""
      
      set pktsInt2 [$qmon2 get-pkts-integrator]
      set suma2 [expr [$pktsInt2 set sum_]]
      set avg_queue2 [expr $suma2/$fin_sim]
      puts [format "N promedio de paquetes en la cola 0----1 = %9.5f paquetes" $avg_queue2]
      puts ""
      
      set pktsInt3 [$qmon3 get-pkts-integrator]
      set suma3 [expr [$pktsInt3 set sum_]]
      set avg_queue3 [expr $suma3/$fin_sim]
      puts [format "N promedio de paquetes en la cola 4----5 = %9.5f paquetes" $avg_queue3]
      puts ""
              
      exec xgraph cola_2_4 -geometry 800x400 &
      exec xgraph cola_3_4 -geometry 800x400 &
      exec xgraph cola_0_1 -geometry 800x400 &
      exec xgraph cola_4_5 -geometry 800x400 &
#     exec nam out.nam &
     
      exit 0

}

proc sendpacket {} {
    global ns fuente InterArrivalTime pktSize 
    set time [$ns now]
    $ns at [expr $time + [$InterArrivalTime value]] "sendpacket"
    set bytes [expr round ([$pktSize value])]
    $fuente send $bytes
}

# se indican las operaciones

$ns at 0 "record"
$ns at 0.0001 "sendpacket"
$ns at [expr $fin_sim+0.01] "finish"

$ns run


